"use strict";
const Path = require("path");
const Generator = require("yeoman-generator");
const yosay = require("yosay");
/**
 * Provides the functionality to generate a node-module written in TypeScript.
 */
class NodeModuleTSGenerator extends Generator {
    /**
     * Initializes a new instance of the `NodeModuleTSGenerator` class.
     *
     * @param args
     * A set of arguments.
     *
     * @param opts
     * A set of options.
     */
    constructor(args, opts) {
        super(args, opts);
        /**
         * The options provided by the user.
         */
        this.extensionConfig = {};
        /**
         * Validates whether the a value is provided.
         *
         * @param {string} value
         * The value that is to be validated.
         *
         * @param {Generator.Answers} answers
         * The answers provided by the user.
         */
        this.ForceInput = (value, answers) => {
            if (value.length > 0) {
                return true;
            }
            else {
                return "Please enter a valid input!";
            }
        };
        this.argument("type", { type: String, required: false });
        this.argument("displayName", { type: String, required: false });
        this.argument("name", { type: String, required: false });
        this.argument("description", { type: String, required: false });
    }
    /**
     * Initializes the generator.
     */
    initializing() {
        for (let key in this.args) {
            switch (key) {
                case "type":
                    this.extensionConfig[key] = "ext-" + this.args[key];
                    break;
                case "displayName":
                case "name":
                case "description":
                    this.extensionConfig[key] = this.args[key];
                    break;
            }
        }
    }
    /**
     * Collects all informations about the module that is to be created.
     */
    prompting() {
        this.log(yosay("Welcome to the TypeScript node-module generator!"));
        let prompts = [
            {
                type: "input",
                name: "destination",
                message: "What directory do you ant to create the package to?",
                default: "./",
                filter: (value, answers) => {
                    this.destination = Path.resolve(process.cwd(), value);
                    return value;
                }
            },
            {
                type: "input",
                name: "displayName",
                message: "What's the display-name of your module?",
                default: (answers) => {
                    return Path.basename(this.destination);
                },
                validate: this.ForceInput
            },
            {
                type: "input",
                name: "name",
                message: "What's the name of your module?",
                default: (answers) => {
                    return answers.displayName.replace(/([a-z])([A-Z])/, "$1-$2").replace(/([A-Z])([A-Z][a-z])/, "$1-$2").toLowerCase();
                },
                validate: this.ForceInput,
                filter: (value, answers) => {
                    return value.replace(/([a-z])([A-Z])/, "$1-$2").replace(/([A-Z])([A-Z][a-z])/, "$1-$2").toLowerCase();
                }
            },
            {
                type: "input",
                name: "description",
                message: "What's the description of your module?"
            },
            {
                type: "input",
                name: "author",
                message: "Please enter your name.",
                default: this.user.git.name,
                store: true
            },
            {
                type: "input",
                name: "authorURL",
                message: "Please enter your homepage.",
                store: true
            }
        ];
        return this.prompt(prompts).then((answers) => {
            Object.assign(this.extensionConfig, answers);
        });
    }
    /**
     * Writes the templates.
     */
    writing() {
        this.destinationRoot(this.destination);
        this.fs.copy(this.templatePath("_.vscode"), this.destinationPath(".vscode"));
        this.fs.copy(this.templatePath("test"), this.destinationPath("test"));
        this.fs.copy(this.templatePath("_.npmignore"), this.destinationPath(".npmignore"));
        this.fs.copy(this.templatePath("_.gitignore"), this.destinationPath(".gitignore"));
        this.fs.copy(this.templatePath("LICENSE"), this.destinationPath("LICENSE"));
        this.fs.copyTpl(this.templatePath("README.md.ejs"), this.destinationPath("README.md"), this.extensionConfig);
        this.fs.copyTpl(this.templatePath("node-module-ts-quickstart.md"), this.destinationPath("node-module-ts-quickstart.md"), this.extensionConfig);
        this.fs.copy(this.templatePath("_tsconfig.json"), this.destinationPath("tsconfig.json"));
        this.fs.copy(this.templatePath("_tslint.json"), this.destinationPath("tslint.json"));
        this.fs.copyTpl(this.templatePath("src/main.ts"), this.destinationPath("src/main.ts"), this.extensionConfig);
        this.fs.copyTpl(this.templatePath("src/tests/main.test.ts"), this.destinationPath("src/tests/main.test.ts"), this.extensionConfig);
        this.fs.copyTpl(this.templatePath("_package.json.ejs"), this.destinationPath("package.json"), this.extensionConfig);
    }
    /**
     * Installs the dependencies.
     */
    install() {
        this.installDependencies({ bower: false, npm: true });
    }
    /**
     * Show some helpful messages after finishing the installation-process.
     */
    end() {
        this.log();
        this.log("Your module \"" + this.extensionConfig.displayName + "\" has been created!");
        this.log();
        this.log("To start editing with Visual Studio Code, use the following commands:");
        this.log();
        this.log("    cd " + this.extensionConfig.destination);
        this.log("    code .");
        this.log();
        this.log("Open node-module-ts-quickstart.md inside the new module for further instructions on how to modify and test your module.");
        this.log();
        this.log();
    }
}
module.exports = NodeModuleTSGenerator;
//# sourceMappingURL=index.js.map